@@ -131,6 +131,9 @@ my $S_IFLNK = 0120000;		# Bit in stat modes field that indicates this
 				# is a symbolic link.
 my $S_IFDIR = 040000;		# This is a directory.
 
+my $is_windows = $^O eq 'cygwin'; # If on windows NT, filenames are case 
+				# insensitive but case preserving.
+
 #
 # All of the information is stored in the structure below.  %file_info
 # is an associative array indexed by the top-level directories.  In
@@ -232,6 +235,10 @@ sub absolute_filename {
   }
 
   my $retstr = join("/", $fileinfo->{SHORTEST_FULLNAME}, reverse @subdirs);
+  $is_windows and $retstr =~ s@^/([A-Za-z]:)@$1@;
+				# Convert /C: to C:.  We converted the other
+				# way so we could use unix file name syntax
+				# everywhere.
   $retstr;
 }
 
@@ -255,6 +262,10 @@ sub absolute_filename_nolink {
 				# Add another directory.
   }
 
+  $is_windows and $ret_str =~ s@^/([A-Za-z]:)@$1@;
+				# Convert /C: to C:.  We converted the other
+				# way so we could use unix file name syntax
+				# everywhere.
   return "/$ret_str";
 }
 
@@ -286,9 +297,15 @@ sub chdir {
     $status = CORE::chdir($_[0]);
   }
 
-  $status ||
-    die ("chdir: can't cd to directory " . $newdir->absolute_filename .
-	 "--$!\n");
+  unless ($status) {
+    if ($newdir->{ALTERNATE_VERSIONS}) { # Was it from a repository?
+      $newdir->mkdir;           # Make it.
+      $status = CORE::chdir($newdir->absolute_filename_nolink);
+    }
+    $status or
+      die ("chdir: can't cd to directory " . $newdir->absolute_filename .
+           "--$!\n");
+  }
 
   publish($newdir);		# Make sure we know about this directory.
 
@@ -400,6 +417,13 @@ sub file_info {
   return $file if ref($file) eq 'FileInfo'; # Don't do anything if we were
 				# already passed a FileInfo structure.
 
+  if ($is_windows) {
+    $file =~ s@^([A-Za-z]):@/$1:@; # If on windows, transform C: into /C: so it
+				# looks like it's in the root directory.
+    $file = lc($file);		# Switch to all lower case to avoid
+				# confounds with mix case.
+  }
+
   my $finfo = ($file =~ s@^/+@@ ? $FileInfo::root :
 		 ($_[1] || $FileInfo::CWD_INFO));
 				# Start at current directory or
@@ -503,6 +527,7 @@ exist, even if there are FileInfo structures for them.
 #
 sub find_file {
   my ($target_fname, $dirinfo) = @_; # Name the arguments.
+  $is_windows and $target_fname = lc($target_fname);
   $dirinfo ||= $FileInfo::root; # Default to root directory if not
 				# specified.
 
@@ -788,6 +813,7 @@ sub read_directory {
 
   foreach (readdir(DIRHANDLE)) {
     next if $_ eq '.' || $_ eq '..'; # Skip the standard subdirectories.
+    $is_windows and $_ = lc($_);
     my $finfo = ($dirinfo->{DIRCONTENTS}{$_} ||=
 		 bless { NAME => $_, ".." => $dirinfo });
 				# Get the file info structure, or make
