@@ -180,6 +180,11 @@ $FileInfo::made_temporary_link = 0; # True if we made any temporary links.
 sub move_or_link_target {
   my ($dest_finfo, $src_finfo) = @_;
 
+  if ($dest_finfo->{DIRCONTENTS}) { # Is this a directory?
+    $dest_finfo->mkdir;         # Just make it, don't soft link to it.
+    return;
+  }
+
   ++$FileInfo::made_temporary_link; # Remember to undo this temporary link.
 
   $dest_finfo->{".."}->mkdir;	# Make the directory (and its parents) if
@@ -451,16 +456,9 @@ sub set_rule {
 	    return;
 	  }
 	  else {
-	    main::print_error("warning: conflicting rules (", $rule->source,
-			      " and ", $oldrule->source, " for target ",
-			      $finfo->name) 
-	      unless $rule->source eq $oldrule->source;
-				# In the linux kernel, several different 
-				# makefiles load the same include file with a
-				# different cwd, and the include file has a
-				# bunch of commands to update files given
-				# their absolute path.
-	    return;
+            main::print_error("warning: conflicting rules (", $rule->source,
+                              " and ", $oldrule->source, " for target ",
+                              $finfo->name);
 	  }
 
 	}			# End if old rule was an explicit rule.
@@ -495,6 +493,11 @@ sub set_rule {
 			      " chosen because it has a shorter chain of inference");
 	    return;
 	  }
+          else {
+            if ($rule->source eq $oldrule->source) {
+              main::print_error("warning: rule ", $rule->source, " produces ", $finfo->name, " in two different ways");
+            }
+          }
 	}
       }
     }
@@ -509,6 +512,8 @@ sub set_rule {
 #    main::print_log(0, $rule->source, " applies to target ", $finfo->name);
 
   $finfo->{RULE} = $rule;	# Store the new rule.
+  $rule->{LOAD_IDX} = $rule->{MAKEFILE}{LOAD_IDX};
+                                # Remember which makefile load it came from.
 
   if (exists $finfo->{BUILD_HANDLE} && !$finfo->{BUILT_WITH_NONDEFAULT}) {
     main::print_error("warning: I became aware of the rule ", $rule->source,
@@ -650,6 +655,7 @@ sub load_build_info_file {
 				# we know whether the build info has changed.
   my $line;
   while (defined($line = <BUILD_INFO>)) {	# Read another line.
+    $line =~ s/\r//;		# Strip out the silly windows-format lines.
     if ($line =~ /(.*?)=(.*)/) { # Check the format.
       my $key = $1;
       my $val = $2;
