@@ -11,6 +11,8 @@ require Exporter;
 @Glob::EXPORT = qw(chdir);	# Force our caller to use our chdir sub that
 				# we inherit from FileInfo.
 @Glob::EXPORT_OK = qw(zglob zglob_fileinfo $allow_dot_files wildcard_action);
+
+my $is_windows = $^O eq 'cygwin'; # Case-insensitive filenames.
 use strict;
 
 =head1 NAME
@@ -105,6 +107,13 @@ sub zglob_fileinfo {
 
   my $is_wildcard = 0;		# We haven't seen a wildcard yet.
 
+  if ($is_windows) {
+    s@^([A-Za-z]):@/$1:@;	# If on windows, transform C: into /C: so it
+				# looks like it's in the root directory.
+    $_ = lc($_);		# Switch to lower case to avoid problems with
+				# mixed case.
+  }
+
   s@^/@@ and $startdir = $FileInfo::root; # If this is a rooted wildcard,
 				# change to the top of the path.  Also,
 				# strip out the leading /.
@@ -293,6 +302,9 @@ sub find_real_subdirs {
   defined($dirstat->[3]) and	# If this directory doesn't exist, then it 
 				# doesn't have subdirectories.
     $expected_subdirs = $dirstat->[3]-2;
+				# Note that if we're on a samba-mounted
+				# file system, $expected_subdirs will be -1
+				# since it doesn't keep a link count.
 
   $expected_subdirs or return (); # Don't even bother looking if this is a
 				# leaf directory.
@@ -308,6 +320,9 @@ sub find_real_subdirs {
 # them are usually editor backups.  Similarly, files with alphabetic
 # extensions (e.g., ".c") are usually not directories.
 #
+# This heuristic doesn't work at all for windows, since it doesn't maintain
+# link counts.
+#
 # On other operating systems, this would be a lot easier since directories
 # often have an extension like ".dir" that uniquely identifies them.
 #
@@ -320,7 +335,7 @@ sub find_real_subdirs {
 #
 # First pass: look for files without extensions and ~ characters.
 #
-  while ($expected_subdirs > 0 &&
+  while ($expected_subdirs != 0 &&
 	 (($dirfilename, $dirfileinfo) = each %{$dirinfo->{DIRCONTENTS}})) {
 				# Look at each file in the directory.
     next if $dirfilename =~ /\~/; # Skip editor backups.
@@ -349,7 +364,7 @@ sub find_real_subdirs {
 # Second pass: look at all files (except the ones we looked at on the previous
 # pass).
 #
-  while ($expected_subdirs > 0 && 
+  while ($expected_subdirs != 0 && 
 	 (($dirfilename, $dirfileinfo) = each %{$dirinfo->{DIRCONTENTS}})) {
 				# Look at each file in the directory that we
 				# haven't looked at before.
@@ -438,6 +453,7 @@ sub wild_to_regex {
 				# parsing.
     my $is_wildcard = 0;	# Haven't seen a wildcard yet.
     my $file_regex = '';	# A regular expression to match this level.
+    $is_windows and $file_regex = '(?i)'; # Make it case insensitive.
     pos($fname) = 0;
       
   parseloop:
@@ -495,6 +511,7 @@ sub wild_to_regex {
   }
 
   $fname =~ s/\\(.)/$1/g;	# Unquote any backslashed characters.
+  $is_windows and return (0, lc($fname));
   return (0, $fname);		# Just return the filename.
 }
 
