@@ -82,7 +82,9 @@ sub expand_text {
   my $ret_str = '';
   pos($_) = 0;			# Suppress a warning message.
 
-  if ($main::rc_substitution) {
+  my $rc_substitution = $ {$self->{PACKAGE} . "::rc_substitution"};
+  !defined($rc_substitution) and $rc_substitution = $main::rc_substitution;
+  if ($rc_substitution) {
 #
 # Code for handling rc-style substitution (the default):
 #
@@ -362,6 +364,13 @@ sub implicitly_load {
 				# try to load from it.  (Directories from
 				# repositories will always be writable since
 				# we're going to create them.)
+#
+# See if this directory or any of its parents is marked for no implicit
+# loading.
+#
+  for (my $pdirinfo = $dirinfo; $pdirinfo; $pdirinfo = $pdirinfo->{".."} || '') {
+    $pdirinfo->{NO_IMPLICIT_LOAD} and return;
+  }
 
   eval { Makefile::load($dirinfo, $dirinfo,
 			$Makefile::global_command_line_vars,
@@ -565,12 +574,14 @@ sub load {
 # environment variables change, it seemed unnecessarily conservative to allow
 # it to do it the old way.
 #
-  wait_for main::build($minfo, 0); # Build the makefile, using what rules we
+  if ($minfo->{NAME} ne 'makepp_default_makefile.mk') {
+    wait_for main::build($minfo, 0); # Build the makefile, using what rules we
 				# know from outside the makefile.  This may
 				# also load it from a repository.
-  delete $minfo->{BUILD_HANDLE}; # Get rid of the build handle, so we avoid
+    delete $minfo->{BUILD_HANDLE}; # Get rid of the build handle, so we avoid
 				# the error message that we built the file
 				# before we saw the rule.
+  }
 
   chdir $mdinfo;		# Get in the correct directory for wildcard
 				# action routines.
@@ -597,8 +608,9 @@ sub load {
 # and reload.  This may leave some bogus rules lying around.  Oh well.
 #
 
-  if ($main::remake_makefiles) { # This often causes problems, so we provide
+  if ($main::remake_makefiles && # This often causes problems, so we provide
 				# a way of turning it off.
+      $minfo->{NAME} ne 'makepp_default_makefile.mk') {
     my $old_n_files = $main::n_files_changed;
     {
       local $main::default_signature_method = $Signature::target_newer::target_newer;
@@ -965,7 +977,8 @@ sub parse_rule {
 
     unless ($foreach) {		# No foreach explicitly specified?
       $foreach = $deps[0];	# Add one, making wildcard from first dep.
-      if ($main::percent_subdirs) { # % searches subdirs?
+      if ($main::percent_subdirs ||
+	  $ {$self->{PACKAGE} . "::percent_subdirs"}) { # % searches subdirs?
 	$foreach =~ s@^\%@**/*@ or # Convert leading % to **/*.
 	  $foreach =~ s/\%/*/;	# Convert nested % to just a *.
       } else {
@@ -1034,12 +1047,14 @@ sub parse_rule {
 
       foreach (@targets) {
 	my $tinfo = main::find_makepp_info(unquote($_), $self->{CWD}); # Access the target object.
+	$tinfo->{PATTERN_LEVEL} = $pattern_level if $was_wildcard_flag;
+				# Remember the pattern level, so we can prevent
+				# infinite loops on patterns.  This must be
+				# set before we call set_rule or we'll get
+				# infinite recursion.
 	$tinfo->set_rule($rule); # Update its rule.  This will be ignored if
 				# it is overriding something we shouldn't
 				# override.
-	$tinfo->{PATTERN_LEVEL} = $pattern_level if $was_wildcard_flag;
-				# Remember the pattern level, so we can prevent
-				# infinite loops on patterns.
 	$was_wildcard_flag or	# If there was no wildcard involved, this is
 				# a candidate for the first target in the file.
 	  $self->{FIRST_TARGET} ||= $tinfo;
@@ -1170,6 +1185,9 @@ sub read_makefile {
     open(MAKEFILE_FH, $Makefile::makefile_name) || 
       die "can't read makefile $Makefile::makefile_name--$!\n";
     $Makefile::makefile_contents = <MAKEFILE_FH>; # Read the whole makefile.
+    $Makefile::makefile_contents =~ s/\r//g;
+				# Strip out those annoying CR characters
+				# which get put in sometimes on windows.
     close MAKEFILE_FH;		# Done with the makefile.
   }
 
