@@ -221,6 +221,15 @@ sub scanner_c_compilation {
   inc_file_loop:
     foreach my $include_fname (@includes) {
 #
+# Check for non-system include files in the same directory as the file itself.
+#
+      my $inc_finfo = file_info($include_fname, $src_info->{".."});
+      if ($inc_finfo->exists_or_can_be_built) {
+	push @source_files, $inc_finfo # Remember it.
+	  unless $all_sources{$inc_finfo}++; # Don't duplicate files.
+	next inc_file_loop;
+      }
+#
 # Check the directories specified in the include path:
 #
       foreach my $incdir (@include_dirs) {
@@ -233,15 +242,6 @@ sub scanner_c_compilation {
       }
 
 #
-# Non-system include files can also be in the same directory as the file itself.
-#
-      my $inc_finfo = file_info($include_fname, $src_info->{".."});
-      if ($inc_finfo->exists_or_can_be_built) {
-	push @source_files, $inc_finfo # Remember it.
-	  unless $all_sources{$inc_finfo}++; # Don't duplicate files.
-	next inc_file_loop;
-      }
-#
 # Do the same thing for the system includes, except that we don't check files
 # that they include--we assume they don't change.  We also don't list them
 # explicitly as dependencies.
@@ -291,7 +291,7 @@ sub scanner_c_compilation {
  libloop:
   foreach my $libname (@libs) {
     foreach my $libdirinfo ($build_cwd, @lib_dirs) {
-      foreach (qw(la a so sa)) { # Common extensions.
+      foreach (qw(la so sa a)) { # Common extensions.
 	my $lib_info = file_info("lib$libname.$_", $libdirinfo);
 	if ($lib_info->exists_or_can_be_built) {
 	  push @obj_files, $lib_info; # Found the library.
@@ -322,6 +322,17 @@ sub scanner_c_compilation {
 sub scanner_none {
 }
 
+#
+# Skip to the next command (after a semicolon or newline) and scan again.
+# This is used to ignore an echo or something like that.
+#
+sub scanner_skip_command {
+  my ($action, $rule) = @_;	# Name the arguments.
+
+  $action =~ s/^.*?[;\n]// and	# Skip to newline or semicolon.
+    $rule->scan_action($action); # Scan the remainder of the action.
+}
+
 #
 # This scanner simply moves to the next word that doesn't begin with
 # - and scans again.
@@ -346,13 +357,17 @@ sub scanner_skip_word {
 # command words.
 #
 %Makesubs::scanners =
-  ("libtool" => \&scanner_libtool,
+  (
 				# These words usually introduce another command
 				# which actually is the real compilation command.
+   "libtool" => \&scanner_skip_word,
    "sh"      => \&scanner_skip_word,
    "purify"  => \&scanner_skip_word,
    "ignore_error" => \&scanner_skip_word,
+   "echo"    => \&scanner_skip_command,
    "noecho"  => \&scanner_skip_word,
+   "fast_cc" => \&scanner_skip_word,
+   "condor_compile" => \&scanner_skip_word,
 
    "cc"      => \&scanner_c_compilation, # These are all the C/C++
    "gcc"     => \&scanner_c_compilation, # compilers I can think of.
@@ -809,6 +824,29 @@ sub f_notdir;
    '^F' => sub { f_notdir f_dependencies },
   );
 
+#
+# Handle the no_implicit_load statement.  This statement marks some
+# directories not to be loaded by the implicit load mechanism, in case
+# there are makefiles there that you really don't want to load.
+#
+sub s_no_implicit_load {
+  my ($text_line, $makefile, $makefile_line) = @_; # Name the arguments.
+
+  my $cwd = $Makesubs::rule ? $Makesubs::rule->build_cwd : $makefile->{CWD};
+				# Get the default directory.
+
+  local $main::implicitly_load_makefiles = 0;
+				# Temporarily turn off makefile loading for
+				# the expansion of this wildcard.
+
+  my @dirs = map { Glob::zglob_fileinfo($_, $cwd) } split(' ', $text_line);
+				# Get a list of things matching the wildcard.
+  foreach my $dir (@dirs) {
+    $dir->is_or_will_be_dir and $dir->{NO_IMPLICIT_LOAD} = 1;
+				# Tag them so they don't load later.
+  }
+}
+
 1;
 
 __DATA__
@@ -964,6 +1002,8 @@ sub s_perl_begin {
     $perl_code .= $line;
   }
 
+  chdir $makefile->{CWD};	# Make sure we're in the correct directory
+				# because some perl code will expect this.
   eval "no strict; package $makefile->{PACKAGE};\n$perl_code";
   if ($@) {			# Try to convert the line number into
 				# something that's more readable.
@@ -1098,81 +1138,6 @@ sub s_unexport {
 # Scanner subroutines:
 #
 
-#
-# Scanning libtool commands is a bit tricky.  Several things we need to do:
-# 1) If a file depends on a xyz.lo, it also depends on .libs/xya.o (or whatever
-#    is specified in the xyz.lo file).
-# 2) .la files cannot be linked in from repositories, because they have
-#    absolute pathnames encoded in them.  Thus we remove repository information
-#    for them.
-#
-sub scanner_libtool {
-  my ($action, $rule) = @_;	# Name the arguments.
-  &scanner_skip_word;		# Call the linker or C compiler scanner, to get
-				# all of the dependencies.
-#
-# Search the dependency list for .lo or .la files:
-#
-  my @handles;
-  my @lofiles;
-  foreach my $finfo (values %{$rule->{ALL_DEPENDENCIES}}) {
-    if ($finfo->{NAME} =~ /\.lo$/) { # Is it a libtool object flie?
-      my $handle = main::build($finfo); # Queue a build for it.
-      $handle and push @handles, $handle; # Remember to wait for it later.
-      push @lofiles, $finfo;	# Remember to look at this later.
-    }
-    elsif ($finfo->{NAME} =~ /\.la$/) {	# Is it a libtool library?
-      delete $finfo->{ALTERNATE_VERSIONS};
-				# Never link these in from repositories.
-    }
-  }
-
-#
-# Search the target list for any .la files:
-#
-  foreach my $finfo (values %{$rule->{ALL_TARGETS}}) {
-    if ($finfo->{NAME} =~ /\.la$/) {
-      delete $finfo->{ALTERNATE_VERSIONS};
-				# Never link these in from repositories.
-    }
-  }
-
-  if (@handles) {
-    my $status = main::wait_for(@handles); # Wait until all the .lo files we 
-				# need actually exist.
-    $status and return;		# Don't bother scanning them if they aren't
-				# correctly build.
-  }	
-  my $build_cwd = $rule->build_cwd; # We need this in several places.
-
-  foreach (@lofiles) {
-    local *LOFILE;		# Make a local file handle.
-    open(LOFILE, $_->absolute_filename) || next; # Read the file.
-    my $line;
-#
-# We're looking for a line in the file that looks like this:
-# pic_object='.libs/xyz.o'
-# or
-# non_pic_object='.libs/xyz.o'
-#
-    while (defined($line = <LOFILE>)) {	# Read a line.
-      if ($line =~ /^\s*(?:non_)?pic_object=\'([^\']+)\'/) { # Found line 
-				# specifying another object?
-	$rule->add_dependency(file_info($1, $build_cwd));
-				# Mark it as a dependency.
-      }
-    }
-    close LOFILE;		# Done with the file.
-  }
-
-#
-# The libtool command file itself is usually also a dependency (it has to be
-# linked in from a repository if it doesn't exist).
-#
-  $action =~ /^\s*(\S+)/;	# Extract the first word.  It's the libtool
-				# commnd.
-  $rule->add_dependency(file_info($1, $build_cwd));
-}
 
 ###############################################################################
 #
@@ -1469,7 +1434,7 @@ sub f_infer_objects {
 	  unless ($source_names{$name}++) { # Did we already know about that source?
 	    if (ref($candidate_objs{$name}) eq 'FileInfo') { # Found a file?
 	      $main::log_level and
-		main::print_log(0, "infer_objects: adding ",
+		main::print_log("infer_objects: adding ",
 				$candidate_objs{$name}->name,
 				" to dependency list because of $_");
 	      push @deps, $candidate_objs{$name}; # Scan for its dependencies.
@@ -1804,6 +1769,10 @@ sub f_MAKE {
 				# a recursive make command.  It doesn't 
 				# actually do anything.
   } else {
+    if (!$main::can_fork) {
+      die "makepp: recursive make without --traditional not supported on this platform\n";
+    }
+
     my $makefile = $_[1];	# Get the makefile we're run from.
 
     my $recursive_makepp = $Config{'bin'} . "/perl " .
