@@ -8,7 +8,6 @@ use FileInfo_makepp;
 use TextSubs;
 use Signature::exact_match;
 
-
 =head1  NAME
 
 Rule -- Stores information about a build rule
@@ -148,6 +147,9 @@ sub find_all_targets_dependencies {
   my $command_string = $makefile->expand_text($self->{COMMAND_STRING},
 					      $self->{RULE_SOURCE});
 				# Get the text of the command.
+  $command_string =~ s/^\s+//;  # Strip out leading and trailing whitespace
+  $command_string =~ s/\s+$//;  # so we don't trigger unnecessary rebuilds
+                                # quite as often.
 
   $self->scan_action($command_string);
 				# Look for any additional dependencies (or
@@ -353,6 +355,12 @@ sub execute_command {
 				# some reason.
     print "$action\n" unless $silent_flag;
     
+    if (!$main::can_fork) {	# Can't fork or exec on windows.
+      system(TextSubs::format_exec_args($action));
+      $? and return ($?/256) || 1; # Quit if an error.
+      next;			# Process the next action.
+    }
+
     if ($error_abort && $action_idx == @actions-1) { # Is this the last action?
       $SIG{__WARN__} = sub {};	# Suppress annyong warning message here if the
 				# exec fails.
@@ -390,6 +398,8 @@ sub execute_command {
     }	
   }
 
+  $main::can_fork or return 0;	# If we didn't fork, we'll always get here.
+
   exit 0;			# If we get here, it means that the last
 				# command in the series was executed with
 				# ignore_error.
