@@ -154,9 +154,12 @@ sub substitute_file {
 sub make_dir {
   my $dirname = '';
   foreach (split(/\//, $_[0])) {
-    $dirname .= "/" . $_;	# Make the new directory name.
-    -d $dirname or
-      mkdir($dirname, 0755);
+    if ($_ ne '') {		# Skip the top level / directory.
+      $dirname .= $_;		# Make the new directory name.
+      -d $dirname or
+	mkdir($dirname, 0755);
+    }
+    $dirname .= '/';
   }
 }
 
