@@ -29,6 +29,7 @@ $old_cwd = cwd;			# Remember where we were so we can cd back
 
 @ARGV or @ARGV = <*.test>;	# Get a list of arguments.
 
+$is_cygwin = $^O eq "cygwin";
 
 $n_failures = 0;
 
@@ -36,6 +37,11 @@ test_loop:
 foreach $tarfile (@ARGV) {
   ($testname = $tarfile) =~ s/\.test$//;
 
+  if ($is_cygwin && $testname =~ /_unix$/) {
+    print "skipping $testname on this platform\n";
+    next;
+  }
+
   -d "tdir" and system("rm -rf tdir"); # Get rid of junk from previous run.
   mkdir "tdir", 0755 || die "$0: can't make directory tdir--$!\n";
 				# Make a directory.
@@ -65,11 +71,13 @@ foreach $tarfile (@ARGV) {
 	open(TFILE, "answers/$tfile") || die "$0: can't open tdir/$tfile--$!\n";
 	$tfile_contents = <TFILE>; # Read in the whole thing.
 	close TFILE;
+	$tfile_contents =~ s/\r//g; # For cygwin, strip out the extra CRs.
 
 	$mtfile = $tfile;		# Get the name of the actual file.
 	open(MTFILE, $mtfile) || die "$mtfile\n";
 	my $mtfile_contents = <MTFILE>; # Read in the whole file.
 	close MTFILE;
+	$mtfile_contents =~ s/\r//g; # For cygwin, strip out the extra CRs.
 	$mtfile_contents eq $tfile_contents or die "$mtfile\n";
       }	
       closedir DIR;
